/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * example.c
 *
 *  Created on: Dec 6, 2010
 *      Author: Timo Minartz
 *
 *
 * Small MPI example how to use the instrumentation.
 *
 * Compile with
 * mpicc example.c -I/sw/eeDaemon/include -L/sw/eeDaemon/lib -leed_client `pkg-config --cflags glib-2.0`
 *
 * and execute with
 * export LD_LIBRARY_PATH=/sw/eeDaemon/lib:$LD_LIBRARY_PATH
 * ./a.out
 */

#include <unistd.h> /* for sleep */

#include "eed.h"
#include "glib.h"
#include "defines.h"

//#include "mpi.h"

void do_some_calc() {
	sleep(5);
}
void do_network_send() {
	sleep(5);
}
void do_network_receive() {
	sleep(5);
}
void do_disk_write() {
	sleep(5);
}

int main(int argc, char** argv) {

	int rank = 0;

	//MPI_Init(&argc,&argv);
	//MPI_Comm_rank(MPI_COMM_WORLD, &rank);

	// doing some calculation. Do not need network and disk...
	g_print("Calling ee_init()\n");
	ee_init(&argc, &argv, 0, rank);

	g_print("Calling ee_cpu_ready()\n");
	ee_dev_mode(MY_CORE_IDS, MODE_MAX);
	g_print("Calling ee_nic_sleep()\n");
	ee_dev_mode(MY_NIC_IDS, MODE_UNUSED);
	g_print("Calling ee_disk_powersave()\n");
	ee_dev_mode(MY_DISK_IDS, MODE_MIN);

	do_some_calc();

	// wake up network to become ready	
	g_print("Calling ee_nic_ready()\n");
	ee_dev_mode(MY_NIC_IDS, MODE_MAX);

	do_some_calc();

	// now network is ready. For our transfer we can reduce the frequency of the cores
	g_print("Calling ee_cpu_powersave()\n");
	ee_dev_mode(MY_CORE_IDS, MODE_MIN);

	if (rank != 0) {
		do_network_send();
		g_print("Calling ee_nic_sleep()\n");
		ee_dev_mode(MY_NIC_IDS, MODE_UNUSED);
	} else {
		// wake up disk for output
		g_print("Calling ee_disk_ready()\n");
		ee_dev_mode(MY_DISK_IDS, MODE_MAX);
		do_network_receive();

		do_disk_write();
	}

	// done. All devices are now sleeping.
	g_print("Calling ee_finalize()\n");
	ee_finalize();

	//MPI_Finalize();

	return 0;
}
