 /*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * eed.h
 *
 *  Created on: Dec 6, 2010
 *      Author: Timo Minartz
 */

#ifndef EED_H_
#define EED_H_
#include "defines.h"

/**
 * Registers the process at the Daemon.
 *
 * All shared resources and the core the process is pinned to are registered for the process,
 * thus is taken into account for mode requests.
 *
 * Registering possibly triggers a mode change corresponding to resource_process_list().
 *
 * If the connection to the server process fails, all following interface calls will be ignored.
 *
 * Tag and Rank only needed for instantiation - all future interface calls are identified by the socket connection.
 *
 * @param argc Pointer to count of commandline args
 * @param argv Pointer to commandline args
 * @param tag Tag for this process, must be the same for all processes in one job (e.g. the torque jobid)
 * @param rank Rank for this process, e.g. the MPI rank
 */
void ee_init(int * argc, char *** argv, int tag, int rank);

/**
 * Distincts the tag by reading the environment variable containing the resource management system jobid.
 * Calls ee_init(). See ee_init() for details.
 *
 * @param argc Pointer to count of commandline args
 * @param argv Pointer to commandline args
 * @param rank Rank for this process, e.g. the MPI rank
 */
void ee_init_rms(int * argc, char *** argv, int rank);

/**
 * Distincts the tag by reading the environment variable containing the resource management system jobid.
 * Calls ee_init() for each possible device with #RMS_RANK. See ee_init() for details.
 *
 * @param argc Pointer to count of commandline args
 * @param argv Pointer to commandline args
 */
void ee_init_rms_all(int * argc, char *** argv);

/**
 * Request Mode for device.
 *
 * If device_id is CORE_ALL_ID, all cores are explicitly registered for the caller!
 *
 * @param device_id Id for this device, e.g. #CORE_ALL_ID (for all cores on the node), #MY_CORE_IDS (for the registered core) or a ID between #CORE_MIN_ID and #CORE_MAX_ID to identify a special device.
 * @param mode_id Id for the desired mode, one out of #MODE_UNUSED, #MODE_MIN, #MODE_MED, #MODE_MAX and #MODE_TURBO
 */
void ee_dev_mode(int device_id, int mode_id);

/**
 * Signalizes the device will be needed to be at least in the specified mode in secs seconds
 *
 * If device_id is CORE_ALL_ID, all cores are explicitly registered for the caller!
 *
 * @param device_id Id for this device, e.g. #CORE_ALL_ID (for all cores on the node), #MY_CORE_IDS (for the registered core) or a ID between #CORE_MIN_ID and #CORE_MAX_ID to identify a special device.
 * @param mode_id Id for the desired mode, one out of #MODE_UNUSED, #MODE_MIN, #MODE_MED, #MODE_MAX and #MODE_TURBO
 * @param secs Seconds afterwards this device should be in #MODE_MAX
 */
void ee_dev_mode_in(int device_id, int mode_id, int secs);

/**
 * Unregisters the process at the Daemon.
 *
 * Unregistering possibly triggers a mode change corresponding to resource_process_list().
 */
void ee_finalize();

/**
 * Unregisters all processes with tag (all processes within a job)
 *
 * Unregistering possibly triggers a mode change corresponding to resource_process_list().
 */
void ee_finalize_all(int tag);

#endif /* EED_H_ */
