/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * Command.h
 *
 *  Created on: Apr 16, 2011
 *  Author: Timo Minartz
 */

#ifndef COMMAND_H_
#define COMMAND_H_

#include <glib.h>
#include <gio/gio.h>
#include <gmodule.h>

/**
 * Encapsulates a GVariant to serialize and deserialize Commands send via network.
 */
typedef struct {
	GVariant * variant;
} Command;

/**
 * Creates a new command without parameters
 * @param format Format for internal gVariant (s), (si), (sil), (sisi), ...
 * @return Pointer to new allocated Command with params if specified
 */
Command* command_new(const gchar* format, ...);

/**
 * Frees the allocated memory by this instance
 * @param command Command to free memory of
 */
void command_unref(Command *command);

/**
 * Prints the Command command in the format of name(param1,param2,...) to stdout.
 * @param command Command to print to stdout
 */
void command_print(Command *command);

/**
 * Serializes the Command and sends it over the GOutputStream stream.
 * Transfered are in the following order the size of the internal GVariant, the size of the GVariantFormat,
 * the GVariantFormatString and the GVariant itself in serialized description.
 *
 * @param stream GOutputStream to write command to
 * @param command Command to write into GOutputStream
 * @return TRUE if write succeeded
 */
gboolean command_send(GOutputStream *stream, Command *command);

/**
 * Reads a serialized Command from the GInputStream and constructs a new Command object.
 *
 * Read are in the following order the size of the internal GVariant, the size of the GVariantFormat,
 * the GVariantFormatString and the GVariant itself in serialized description.
 *
 * @param stream GInputStream to read command form
 * @param command New instance of Command based on serialized representation
 * @return TRUE if read succeeded
 */
gboolean command_receive(GInputStream *stream, GCancellable *cancellable,
		Command **command);

/**
 * Returns the name of the Command.
 * @param command Command to get name of
 * @return Name of the Command command
 */
gchar* command_get_name(Command *command);

gchar* command_get_string(Command *command);

/**
 * Executes the Command in the specified GModule.
 * Parses the internal arguments and constructs the prototype for the module to call.
 * @param command Command to be executed
 * @param module GModule containing symbol for Command command
 * @return True if execution succeeded
 */
gboolean command_execute(Command *command, GModule *module);

#endif /* COMMAND_H_ */
