/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * Client.c
 *
 *  Created on: Dec 6, 2010
 *      Author: Timo Minartz
 */

#include "Client.h"
#include "print.h"

#include <string.h> /* for strcmp */

void process_command_queue(Client * client) {

	debug(2, "Start processing command queue.\n");

	while (1) {

		Command * command = (Command *) g_async_queue_pop(client->cmd_queue);

		if (command != NULL) {

			gchar * name = command_get_name(command);

			if (strcmp(name, TERMINATE_CLIENT_THREAD_MSG) == 0) {
				command_unref(command);
				g_free(name);
				break;
			}



			debug(3, "Sending command %s...\n", name);
			if (!command_send(client->stream, command)) {
				debug(3, "Sending command %s...Failed.\n", name);
				command_unref(command);
				g_free(name);
				break;
			} else debug(3, "Sending command %s...Done.\n", name);

			command_unref(command);
			g_free(name);

		}
	}

	debug(1, "Finished processing command queue.\n");
}

void client_send_command(Client * client, Command * command) {

	if (client != NULL) {
		g_async_queue_push(client->cmd_queue, (gpointer) command);
	} else {
		error("Command queue not initialized - Maybe multiple processes using library compiled without MPI support?\n");
	}
}

Client * client_new(gint port) {

	g_type_init();

	Client * client = g_malloc(sizeof(Client));

	client->port = port;

	client->socket = g_socket_client_new();

	client->conn = g_socket_client_connect_to_host(client->socket, "localhost",
			client->port, NULL, NULL);

	if (client->conn == NULL) {
		gchar hostname[10];
		gethostname(hostname, 10);
		error("Connection failed. Maybe eeDaemon not started on host %s?\n", hostname);
		g_object_unref(client->socket);
		g_free(client);
		return NULL;
	}

	client->stream = g_io_stream_get_output_stream((GIOStream*) client->conn);

	if (client->stream == NULL) {
		error("Creating output stream failed.\n");
		g_object_unref(client->conn);
		g_object_unref(client->socket);
		g_free(client);
		return NULL;
	}

	// initialize asynchronous queue
	client->cmd_queue = g_async_queue_new();

	// start thread for processing the incoming messages from program side

	client->thread
			= g_thread_create((GThreadFunc) & process_command_queue, (gpointer) client, TRUE, NULL);

	registerThread(client->thread, "client-worker");

	debug(2, "Opened socket and created worker thread.\n");

	return client;

}

void client_unref(Client * client) {

	if (client != NULL) {

		Command * command = command_new("(s)", TERMINATE_CLIENT_THREAD_MSG);

		g_async_queue_push(client->cmd_queue, (gpointer) command);

		g_thread_join(client->thread);

		debug(2, "Worker thread joined.\n");

		g_async_queue_unref(client->cmd_queue);

		g_object_unref(client->conn);

		g_object_unref(client->socket);

		debug(2, "Socket closed.\n");

		g_free(client);

	}

}
