/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * Client.h
 *
 *  Created on: Dec 6, 2010
 *      Author: Timo Minartz
 */

#ifndef CLIENT_H_
#define CLIENT_H_

#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>

#ifdef HAVE_MPI
#include "mpi.h"
#endif

#include "Command.h"

/*
 * Asynchronous queue for sending the messages via socket
 */
#define TERMINATE_CLIENT_THREAD_MSG "terminate_now"

typedef struct {
	gint port;
	GSocketClient* socket;
	GSocketConnection* conn;
	GOutputStream* stream;
	GAsyncQueue* cmd_queue;
	GThread *thread;
} Client;

/**
 * Create and connect the socket. A worker thread is initialized which will process
 * all messages put in the queue via client_send_command().
 *
 * @param port Socket port
 * @return new Client object or NULL in case of error
 */
Client * client_new(gint port);

/**
 * Put a new Command in the queue for the worker thread
 *
 * @param client Client to send Command from
 * @param command The Command to send to the server daemon
 */
void client_send_command(Client * client, Command * command);

/**
 * Closes the connection and finish the worker thread
 */
void client_unref(Client * client);

#endif /* CLIENT_H_ */
