/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * client_main.c
 *
 *  Created on: Aug 5, 2011
 *      Author: Timo Minartz
 */

#include "eed.h"
#include <glib.h>
#include "print.h"
#include "defines.h"

int main(int argc, char** argv) {

#ifdef G_THREADS_ENABLED
	registerThread(g_thread_self(), "main");
#endif

	gint tag = -1;
	gint rank = -1;
	gchar* application = "";

	/**
	 * for processing commandline args
	 */
	GOptionEntry entries[] = {

	/* verbose options */
	{ "verbose", 'v', 0, G_OPTION_ARG_INT, &verbose, "Be verbose", NULL },
	/* init */
	{ "init", 'i', 0, G_OPTION_ARG_STRING, &application,
			"Initialize daemon for this application", "name" },
	/* rank, optional for init */
	{ "rank", 'r', 0, G_OPTION_ARG_INT, &rank, "Rank to register (optional)",
			"rank" },
	/* finalize */
	{ "finalize", 'f', 0, G_OPTION_ARG_INT, &tag, "Finalize this tag", "tag" },

	{ NULL } };

	/* check commandline args */

	GError *error = NULL;
	GOptionContext *context;

	context = g_option_context_new("- eeClust Client");
	g_option_context_add_main_entries(context, entries, NULL);

	if (!g_option_context_parse(context, &argc, &argv, &error)) {
		error("option parsing failed: %s\n", error->message);
		return 1;
	}

	g_option_context_free(context);

	info("Be verbose.\n");

	// tag must be provided
	if (strcmp(application, "") == 0 && tag == -1) {
		error("Please select an valid set of options\n");
		return 2;
	}

	if (strcmp(application, "") != 0) {

		gint ee_argc = 1;
		gchar ** ee_argv = g_malloc(sizeof(gchar*));
		argv[0] = application;
		if (rank == -1) {
			info("Registering app %s...\n", application);
			ee_init_rms_all(&ee_argc, &ee_argv);
		} else {
			info("Registering app %s for rank %d...\n", application, rank);
			ee_init_rms(&ee_argc, &ee_argv, rank);
		}

	} else if (tag >= 0) {
		info("Finalizing tag %d...\n", tag);
		ee_finalize_all(tag);
	}

	return 0;

}
