/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * idle.c
 *
 *  Created on: Nov 2, 2011
 *      Author: Timo Minartz
 */
#include <glib.h>
#include <unistd.h> /* for sleep */
#include "eed.h"
#include <sys/time.h>
#include <stdint.h>

/**
 * Creates a dev_ready_in call to eeDaemon for each device for specified seconds.
 * This application is an idle job, which could be created by a mesource management system to reduce power consumption while idle.
 */
gint main(gint argc, gchar ** argv) {

	gint secs = 0;

	/**
	 * for processing commandline args
	 */
	GOptionEntry entries[] = {

	/* idle for */
	{ "idle-for", 'i', 0, G_OPTION_ARG_INT, &secs, "Time in seconds devices are assumed to be idle", "secs" },

	{ NULL } };

	/* check commandline args */

	GError *error = NULL;
	GOptionContext *context;

	context = g_option_context_new("- Puts all devices for secs seconds into MODE_UNUSED. Blocking.");
	g_option_context_add_main_entries(context, entries, NULL);

	if (!g_option_context_parse(context, &argc, &argv, &error)) {
		g_printerr("option parsing failed: %s\n", error->message);
		return 1;
	}

	g_option_context_free(context);

	if(secs <= 0) {
		g_printerr("Please specify an idle time > 0\n");
		return 2;
	}

	ee_init_rms(&argc, &argv, 1);

	ee_dev_mode_in(CORE_ALL_ID, MODE_MAX, secs);
	ee_dev_mode_in(NIC_ALL_ID, MODE_MAX, secs);
	ee_dev_mode_in(DISK_ALL_ID, MODE_MAX, secs);

	sleep(secs);

	ee_finalize();

	return 0;
}
