/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * print.h
 *
 * This header defines makros and methods to print from parallel programs to stdout.
 *
 *  Created on: Feb 4, 2011
 *  Author: Timo Minartz
 */

#ifndef PRINT_H_
#define PRINT_H_

#include <glib.h>

#ifdef HAVE_MPI
#include "mpi.h"
#include <unistd.h> /* for gethostname */

static gchar host[10] = "localhost";
#endif

#ifdef G_THREADS_ENABLED
#include <string.h> /* for strcmp */
static GHashTable * threads = NULL;
gchar * getThreadName(GThread * thread);
void registerThread(GThread * thread, gchar * name);
void clearThreads();
#endif

gint print_debug_messages;
gboolean verbose;

#ifdef DEBUG
// use debugging information: File and Line
#define error(...)	{g_printerr("%s:%d - ",__FILE__, __LINE__); g_printerr(__VA_ARGS__); }
#else
// only print error message
#define error(...)	{g_printerr(__VA_ARGS__); }
#endif

#ifdef HAVE_MPI
#ifdef G_THREADS_ENABLED
//MPI and GTHREADS
#define print(...) 	{gethostname(host, 10); g_print("[%s: %s] ", getThreadName(g_thread_self()), host); g_print(__VA_ARGS__);}
#else
//MPI, but no GTHREADS
#define print(...) 	{gethostname(host, 10); g_print("[%s]: ", host); g_print(__VA_ARGS__);}
#endif /* G_THREADS_ENABLED */
#else
#ifdef G_THREADS_ENABLED
// no MPI, but GTHREADS
#define print(...) 	{if (getThreadName(g_thread_self()) != NULL) { g_print("[%s] ", getThreadName(g_thread_self())); } g_print(__VA_ARGS__);}
#else
// whether MPI than GTHREADS
#define print(...) 	{g_print(__VA_ARGS__);}
#endif /* G_THREADS_ENABLED */
#endif /* HAVE_MPI */


#define info(...) 	if(verbose || print_debug_messages > 0) {print(__VA_ARGS__); }

#ifdef DEBUG
// only include debugging messages if compiled with debugging options

#define debug(level, ...) 	if (print_debug_messages >= level) { print(__VA_ARGS__); }

#else
// discard any debugging messages...
#define debug(...) ;
#endif

/*
 * g_printerr("\033[22;31m", __VA_ARGS__); -> red output
\033[22;30m is black
\033[22;31m is red
\033[22;32m is green
yellow
blue
purple
turkis
grey
light white
*/

#endif /* PRINT_H_ */
