/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * ConstrainedResource.h
 *
 *  Created on: Jul 12, 2011
 *      Author: Timo Minartz
 */

#ifndef CONSTRAINEDRESOURCE_H_
#define CONSTRAINEDRESOURCE_H_

#include <glib.h>
#include "Resource.h"

typedef struct {
	GList * resources;
	glong interval;
	gboolean finalize;
} ConstrainedResource;

ConstrainedResource* constrained_resource_new(glong interval);

void constrained_resource_unref(ConstrainedResource * cr);

void constrained_resource_register(ConstrainedResource * cr, Resource * resource);

void constrained_resource_unregister(ConstrainedResource * cr, Resource * resource);

/**
 * Finds the minimum mode this ConstrainedResource should operate in and updates requests.
 * Internally, it is iterated over all Resources registered for this ConstrainedResource.
 * For each resource, resource_process_list() is called.
 * @param cr ConstrainedResource to process list of
 * @param time_passed Time in microseconds passed since the last invocation of this method
 * @return minimum Mode this ConstrainedResource should operate in
 */
Mode constrained_resource_process_list(ConstrainedResource *cr, glong time_passed);

void constrained_resource_process(ConstrainedResource * cr);

#endif /* CONSTRAINEDRESOURCE_H_ */
