/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * Disk.h
 *
 *  Created on: Feb 22, 2011
 *  Author: Timo Minartz
 */

#ifndef DISK_H_
#define DISK_H_

#include <glib.h>

#include "defines.h"

/**
 * This struct stores the information about a Disk.
 */
typedef struct {
	/**
	 * corresponding device
	 */
	gchar * device;
	/**
	 * current Mode of the disk
	 */
	Mode mode;
	/**
	 * Time in microseconds needed for this Disk to be in MODE_MAX
	 */
	glong startup_time;
	/**
	 * String list containing the remount commands for this device with disabled pdflush etc.
	 */
	GList* remount_ee_commands;
	/**
	 * String list containing the remount commands for this device
	 */
	GList* remount_commands;
} Disk;

/**
 * Returns the actual Mode of the Disk
 * @param disk Disk to get Mode of
 * @return Mode of the Disk
 */
Mode disk_get_mode(Disk *disk);

/**
 * Returns the actual State of the Disk
 * @param disk Disk to get State of
 * @return State of the Disk: STATE_UNKNOWN, STATE_STANDBY, STATE_SPINDOWN, STATE_SPINUP, STATE_IDLE, STATE_ACTIVE_IDLE
 */
State disk_get_state(Disk *disk);

/**
 * Switches the Disk to the desired mode. If already in this mode, nothing happens. Otherwise the switch is initiated using hdparm.
 * @param disk Disk to operate on
 * @param mode Mode to switch to
 */
void disk_switch_to_mode(Disk * disk, Mode mode);

/**
 * Constructor, creates a new Disk with the standard device /dev/sda and 200 microseconds startup time
 * @return a new Disk
 */
Disk* disk_new();

/**
 * Constructor, creates a new Disk.
 * The internalDisk is initiated.
 * The internal thread for the ResourceManager is created.
 * @param device internal device of the disk
 * @param startup_time time needed to startup the device in microseconds
 * @return a new Disk or NULL if initialization fails
 */
Disk* disk_new_dev(gchar *device, glong startup_time);

/**
 * Destructor, frees the allocated memory
 * @param disk Disk to free
 */
void disk_unref(Disk *disk);

#endif /* DISK_H_ */
