/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * Nic.h
 *
 *  Created on: Feb 22, 2011
 *  Author: Timo Minartz
 */

#ifndef NIC_H_
#define NIC_H_

#include <glib.h>

#include <netinet/in.h> /* for ifru */
#include <linux/if.h> /* for ifreq */
#include <linux/ethtool.h> /* for ethtool_cmd */

#include "defines.h"

/**
 * This struct stores the information about a network card
 */
typedef struct {
	/**
	 * corresponding device
	 */
	gchar* device;
	/**
	 * possible speeds of this device
	 */
	gint* speed;
	/**
	 * count speeds
	 */
	gint count_speed;
	/**
	 * current Mode of the disk
	 */
	Mode mode;
	/**
	 * Socket to communicate with nic over ioctl
	 */
	int sock;
	/**
	 * Structs for generating ethtool like commands
	 */
	struct ifreq ifr;
	struct ethtool_cmd edata;
	/**
	 * Time in microseconds needed for a Nic to be in MODE_MAX
	 */
	glong startup_time;
} Nic;

/**
 * Returns the actual Mode of the Nic
 * @param nic Nic to get Mode of
 * @return Mode of the Nic
 */
Mode nic_get_mode(Nic *nic);

/**
 * Returns the actual State of the Nic
 * @param nic Nic to get State of
 * @return State of the Nic: 1000, 100 or 10 MBit
 */
State nic_get_state(Nic *nic);

/**
 * Switches the Nic to the desired mode. If already in this mode, nothing happens. Otherwise the switch is initiated using ethtool_cmd.
 * @param nic Nic to operate on
 * @param mode Mode to switch to
 */
void nic_switch_to_mode(Nic * nic, Mode mode);


/**
 * Constructor, creates a new Nic with the standard device eth0 and a startup time of 200 milliseconds
 * @return a new Nic
 */
Nic* nic_new();

/**
 * Constructor, creates a new Nic.
 * The internal thread for the ResourceManager is created.
 * @param device internal device of the Nic
 * @param startup_time time needed to startup the device in milliseconds
 * @return a new Nic
 */
Nic* nic_new_dev(gchar * device, glong startup_time);

/**
 * Destructor, frees the allocated memory
 * @param nic Nic to free
 */
void nic_unref(Nic *nic);

#endif /* NIC_H_ */
