/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * ResourcesTable.h
 *
 *  Created on: Jul 12, 2011
 *      Author: Timo Minartz
 */

#ifndef RESOURCESTABLE_H_
#define RESOURCESTABLE_H_

#include <glib.h>
#include "Resource.h"
#include "ConstrainedResource.h"

#ifdef HAVE_HDTRACE
#include "hdStats.h"
//#else
//#define hdStatsGroup gchar
#endif

typedef struct {
	GHashTable * resources;
	GList * constrained_resources;
	gint registrations;
	GThread ** processing_threads;
	gint count_resources;
} ResourcesTable;

ResourcesTable* resources_table_new();

void resources_table_add(ResourcesTable * rt, Resource * resource);

void resources_table_add_constrained_resource(ResourcesTable * rt,
		ConstrainedResource * cr);

void resources_table_delete_resource(ResourcesTable * rt, Resource * resource);

/**
 * Registers resources for combination of tag and rank.
 * All shared resources are registered (Disk and Nic) and the Core with the corresponding ID.
 *
 * @param rt ResourcesTable to register process in
 * @param tag Tag to register
 * @param rank Rank to register
 * @param core_id Register Core with this Core ID
 */
void resources_table_register(ResourcesTable * rt, gint tag, gint rank, gint core_id);
void resources_table_unregister(ResourcesTable *rt, gint tag, gint rank);
void resources_table_unregister_tag(ResourcesTable *rt, gint tag);

/**
 * Return Resource with specified resource ID
 *
 * @param rt ResourcesTable to search in
 * @param id ID to search for between MIN_ID and MAX_ID
 */
Resource * resources_table_get_resource_by_id(ResourcesTable *rt, gint id);

/**
 * Creates list with all resources from user identified with tag and rank and specified type
 *
 * @param rt ResourcesTable to search in
 * @param tag Usertag
 * @param rank Rank
 * @param type One id of out of #CORE_ID, #NIC_ID or #DISK_ID
 */
GList* resources_table_get_resources_by_user_and_type(ResourcesTable *rt, gint tag, gint rank, gint id);

/**
 * Creates list with all resources from user identified with tag and rank.
 *
 * @param rt ResourcesTable to search in
 * @param tag Usertag
 * @param rank Rank
 */
GList* resources_table_get_resources_by_user(ResourcesTable *rt, gint tag, gint rank);

/**
 * Creates list with all resources from type id.
 * If provided id is e.g. a core id, a list containing all resources from type core is returned.
 *
 * @param rt ResourcesTable to search in
 * @param id ID for type
 */
GList* resources_table_get_resources_by_id(ResourcesTable *rt, gint id);

gint resources_table_get_count_registrations(ResourcesTable * rt);
gint resources_table_get_count_resources(ResourcesTable *rt);

void resources_table_start_process_resources(ResourcesTable *rt);
void resources_table_stop_process_resources(ResourcesTable *rt);

void resources_table_unref(ResourcesTable * rt);
void resources_table_unref_full(ResourcesTable * rt);

#ifdef ENABLE_TRACE
void resources_table_init_tracing(ResourcesTable * rt, hdStatsGroup * group);
void resources_table_do_tracing(ResourcesTable * rt, hdStatsGroup * group);
#endif

#endif /* RESOURCESTABLE_H_ */
