/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * Server.h
 *
 *  Created on: Jul 14, 2011
 *  Author: Timo Minartz
 */

#ifndef SERVER_H_
#define SERVER_H_

#include <glib.h>
#include <gio/gio.h>

/**
 * A Server object waits for new socket connections. For every connection
 * a new thread is spawned, which processes the received Commands calling the corresponding methods.
 */
typedef struct {
	/**
	 * Socket port server listens to
	 */
	gint port;
	/**
	 * Count of existing instances of this server
	 */
	gint instances;
	/**
	 * Thread pool for all workers
	 */
	GThreadPool * workerPool;
	/**
	 * Indicates the termination of the server to all working threads
	 */
	gboolean terminate_server;
	/**
	 * Used to cancel blocking waits for inputs in cases of termination
	 */
	GCancellable * cancellable;
} Server;

/**
 * Returns a pointer to a Server listening on Port port. If no Server exists, a new one is initialized.
 * Otherwise, a pointer to the already existing instance is returned.
 * @param port Port to listen on
 * @return Pointer to a Server
 */
Server * server_new(gint port);

/**
 * Opens the socket and starts listening. For each connection a new thread is spawned to process the input.
 * This method is blocking until server_stop() is called or SIGINT or SIGTERM are raised.
 * @return Successful server run
 */
gboolean server_run();

/**
 * Stops the running server. This method is not blocking.
 */
void server_stop();

/**
 * Decreases the pointer counter. If the last pointer, method is blocking until all tasks are finished.
 * Dereferences the Server object.
 * @param server Server object
 */
void server_unref(Server * server);

#endif /* SERVER_H_ */
