/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * main.c
 *
 *  Created on: Jul 13, 2011
 *      Author: Timo Minartz
 */

#include "Server.h"
#include "print.h"
#include "defines.h"
#include <glib.h>

gchar * config_file_path = NULL;

int main(int argc, char** argv) {

#ifdef G_THREADS_ENABLED
	registerThread(g_thread_self(), "main");
#endif

	/**
	 * for processing commandline args
	 */
	GOptionEntry entries[] = { { "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose, "Be verbose", NULL },
			{ "config-file", 'c', 0, G_OPTION_ARG_FILENAME, &config_file_path, "Read config file for hardware configuration from path. Default is /etc/eedaemon.conf.", "path" },
			{ "debug", 'd', 0, G_OPTION_ARG_INT, &print_debug_messages, "Print debugging messages to stdout. Level 0 (default) disables debugging, Level 3 generates lots of messages, including tracing messages", "level"},
			{ NULL } };

	/* check commandline args */

	GError *error = NULL;
	GOptionContext *context;

	context = g_option_context_new("- eeClust Server Daemon");
	g_option_context_add_main_entries(context, entries, NULL);

	if (!g_option_context_parse(context, &argc, &argv, &error)) {
		error("option parsing failed: %s\n", error->message);
		return 1;
	}

	g_option_context_free(context);

	debug(1, "Debuglevel: %d.\n", print_debug_messages);

	if(config_file_path == NULL) {
		// use default
		config_file_path = g_strdup(DEFAULT_CONFIG_FILE);
	}

	debug(1, "Using config file: %s\n", config_file_path);

	Server * server = server_new(1313);

	gint ret = server_run();

	server_unref(server);

	g_free(config_file_path);

	return ret;
}
