/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * CommandTest.c
 *
 *  Created on: Apr 18, 2011
 *  Author: Timo Minartz
 */

#include "Command.h"
#include "CommandTest.h"
#include <string.h>
#include <glib.h>

#define SOCKET_PORT 1313

/*
 * Implement manual type string parsing for all known types from client_eed.c:
 *
 * eed_send_command(command_new("(siisi)", "ee_init", tag, rank, prgname, core_id));
 * eed_send_command(command_new("(siiii)", "ee_dev_mode", device_id, mode_id, tag, rank));
 * eed_send_command(command_new("(siiii)", "ee_dev_idle_for", device_id, secs, tag, rank));
 * eed_send_command(command_new("(siiii)", "ee_dev_ready_in", device_id, secs, tag, rank));
 * eed_send_command(command_new("(sii)", "ee_finalize", tag, rank));
 * eed_send_command(command_new("(si)", "ee_finalize_all", tag));
 */

gint tag = 1313;
gint rank = 0;
gint device_id = 17;
gint mode_id = 13;
gint secs = 19;
gint core_id = 7;
gchar * prgname = "prgname";

void client() {

	GSocketClient* socket_client;
	GSocketConnection* conn;
	GOutputStream* stream;

	socket_client = g_socket_client_new();

	conn = g_socket_client_connect_to_host(socket_client, "localhost",
			SOCKET_PORT, NULL, NULL);

	g_assert(conn != NULL);

	stream = g_io_stream_get_output_stream((GIOStream*) conn);

	g_assert(stream != NULL);

	/*
	 * eed_send_command(command_new("(siisi)", "ee_init", tag, rank, prgname, core_id));
	 *
	 */

	Command *command = command_new("(siisi)", "ee_init", tag, rank, prgname, core_id);

	command_send(stream, command);

	command_unref(command);

	/*
	 * eed_send_command(command_new("(sii)", "ee_finalize", tag, rank));
	 */

	command = command_new("(sii)", "ee_finalize", tag, rank);

	command_send(stream, command);

	command_unref(command);

	/*
	 * eed_send_command(command_new("(siiii)", "ee_dev_mode", device_id, mode_id, tag, rank));
	 * eed_send_command(command_new("(siiii)", "ee_dev_idle_for", device_id, secs, tag, rank));
	 * eed_send_command(command_new("(siiii)", "ee_dev_ready_in", device_id, secs, tag, rank));
	 */

	command = command_new("(siiii)", "ee_dev_mode", device_id, mode_id, tag,
			rank);

	command_send(stream, command);

	command_unref(command);

	/* eed_send_command(command_new("(si)", "ee_finalize_all", tag)); */

	command = command_new("(si)", "ee_finalize_all", tag);

	command_send(stream, command);

	command_unref(command);

	// results in GLib-GIO-CRITICAL **: g_output_stream_close: assertion `G_IS_OUTPUT_STREAM (stream)' failed
	//	g_object_unref(stream); GLib Documentation says: Do not free....

	g_object_unref(conn);

	g_object_unref(socket_client);
}

void server() {
	GSocketListener * listen;
	GSocketConnection * conn;
	GInputStream * stream;
	GCancellable * cancellable;
	GError *error = NULL;
	gchar* line;

	// create an cancellable object for the the stream_read_line
	cancellable = g_cancellable_new();

	listen = g_socket_listener_new();
	g_socket_listener_add_inet_port(listen, SOCKET_PORT, NULL, NULL);

	conn = g_socket_listener_accept(listen, NULL, cancellable, &error);

	if (error != NULL && error->code == G_IO_ERROR_CANCELLED) {
		if (conn != NULL)
			g_object_unref(conn);

		g_error_free(error);
		g_assert_not_reached();
	}

	stream = g_io_stream_get_input_stream((GIOStream*) conn);

	Command *command = NULL;

	gchar * type_string;
	gchar * name;
	gint param1;
	gint param2;
	gint param3;
	gchar * param3s;
	gint param4;

	/* eed_send_command(command_new("(siisi)", "ee_init", tag, rank, prgname, core_id)); */

	command_receive(stream, cancellable, &command);

	type_string = (gchar*) g_variant_get_type_string(command->variant);

	g_variant_get(command->variant, type_string, &name, &param1, &param2,
			&param3s, &param4);

	g_assert_cmpstr(name, ==, "ee_init");
	g_assert_cmpint(param1, ==, tag);
	g_assert_cmpint(param2, ==, rank);
	g_assert_cmpstr(param3s, ==, prgname);
	g_assert_cmpint(param4, ==, core_id);

	g_free(name);
	g_free(param3s);

	command_unref(command);

	/* eed_send_command(command_new("(sii)", "ee_finalize", tag, rank)); */

	command_receive(stream, cancellable, &command);

	type_string = (gchar*) g_variant_get_type_string(command->variant);

	g_variant_get(command->variant, type_string, &name, &param1, &param2);

	g_assert_cmpstr(name, ==, "ee_finalize");
	g_assert_cmpint(param1, ==, tag);
	g_assert_cmpint(param2, ==, rank);

	g_free(name);

	command_unref(command);

	/* eed_send_command(command_new("(siiii)", "ee_dev_mode", device_id, mode_id, tag, rank)); */

	command_receive(stream, cancellable, &command);

	type_string = (gchar*) g_variant_get_type_string(command->variant);

	g_variant_get(command->variant, type_string, &name, &param1, &param2,
			&param3, &param4);

	g_assert_cmpstr(name, ==, "ee_dev_mode");
	g_assert_cmpint(param1, ==, device_id);
	g_assert_cmpint(param2, ==, mode_id);
	g_assert_cmpint(param3, ==, tag);
	g_assert_cmpint(param4, ==, rank);

	g_free(name);

	command_unref(command);

	/* eed_send_command(command_new("(si)", "ee_finalize_all", tag)); */

	command_receive(stream, cancellable, &command);

	type_string = (gchar*) g_variant_get_type_string(command->variant);

	g_variant_get(command->variant, type_string, &name, &param1);

	g_assert_cmpstr(name, ==, "ee_finalize_all");
	g_assert_cmpint(param1, ==, tag);

	g_free(name);

	command_unref(command);

	// results in GLib-GIO-CRITICAL **: g_output_stream_close: assertion `G_IS_OUTPUT_STREAM (stream)' failed
	//	g_object_unref(stream); GLib Documentation says: Do not free....

	g_object_unref(conn);

	g_object_unref(listen);

	g_object_unref(cancellable);
}

void server_multiple_clients() {

	GSocketListener * listen;
	GSocketConnection * conn[2];
	GInputStream * stream[2];
	GCancellable * cancellable;
	GError *error = NULL;

	// create an cancellable object for the the stream_read_line
	cancellable = g_cancellable_new();

	listen = g_socket_listener_new();

	for (gint i = 0; i < 2; ++i) {

		g_socket_listener_add_inet_port(listen, SOCKET_PORT, NULL, NULL);

		conn[i] = g_socket_listener_accept(listen, NULL, cancellable, &error);

		if (error != NULL && error->code == G_IO_ERROR_CANCELLED) {
			if (conn[i] != NULL)
				g_object_unref(conn[i]);

			g_error_free(error);
			g_assert_not_reached();
		}

		stream[i] = g_io_stream_get_input_stream((GIOStream*) conn[i]);

	}

	Command * command = NULL;

	for (gint i = 0; i < 2; ++i) {
		for (gint j = 0; j < 4; ++j) {
			command_receive(stream[i], cancellable, &command);
			command_unref(command);
		}

		g_object_unref(conn[i]);

	}

	// results in GLib-GIO-CRITICAL **: g_output_stream_close: assertion `G_IS_OUTPUT_STREAM (stream)' failed
	//	g_object_unref(stream); GLib Documentation says: Do not free....

	g_object_unref(listen);

	g_object_unref(cancellable);
}

void test_command_communication(void) {

	if (!g_thread_get_initialized())
		g_thread_init(NULL);

	g_type_init();

	GThread *master;
	GThread *slave;

	slave = g_thread_create((GThreadFunc) & client, NULL, TRUE, NULL);
	master = g_thread_create((GThreadFunc) & server, NULL, TRUE, NULL);

	g_thread_join(slave);
	g_thread_join(master);

}

void test_command_multiple_clients(void) {
	if (!g_thread_get_initialized())
		g_thread_init(NULL);

	g_type_init();

	GThread *master;
	GThread *slave1;
	GThread *slave2;

	slave1 = g_thread_create((GThreadFunc) & client, NULL, TRUE, NULL);
	slave2 = g_thread_create((GThreadFunc) & client, NULL, TRUE, NULL);
	master
			= g_thread_create((GThreadFunc) & server_multiple_clients, NULL, TRUE, NULL);

	g_thread_join(slave1);
	g_thread_join(slave2);
	g_thread_join(master);

}

void test_command_get_string(void) {

	gchar * name = "ee_init";

	Command *command = command_new("(siisi)", name, tag, rank, prgname, core_id);

	gint len = strlen(name) + 30 + strlen(prgname);

	gchar * result = g_malloc(len);

	g_snprintf(result, len-1, "%s(%d,%d,%s,%d)", name, tag, rank, prgname, core_id);

	g_assert_cmpstr(command_get_string(command), ==, result);

	g_free(result);

	g_assert_cmpint(g_str_has_prefix(command_get_string(command), "ee_"), ==, TRUE);

	g_assert_cmpint(!g_str_has_prefix(command_get_string(command), "ee_"), ==, FALSE);

	command_unref(command);

}

void test_command_get_name(void) {

	gchar * name = "ee_init";

	Command *command = command_new("(siisi)", name, tag, rank, prgname, core_id);

	g_assert_cmpstr(command_get_name(command), ==, name);

	command_unref(command);

}

void test_command_execute(void) {

	/* open function module in main context */

	GModule *module = g_module_open(NULL, G_MODULE_BIND_LAZY);

	g_assert(module != NULL);

	/*
	 * eed_send_command(command_new("(siis)", "ee_init", tag, rank, prgname));
	 */

	Command *command = command_new("(siisi)", "ee_init", tag, rank, prgname, core_id);

	gboolean ret = command_execute(command, module);

	g_assert_cmpint(ret, ==, TRUE);

	command_unref(command);

	/*
	 * eed_send_command(command_new("(siiii)", "ee_dev_mode", device_id, mode_id, tag, rank));
	 * eed_send_command(command_new("(siiii)", "ee_dev_idle_for", device_id, secs, tag, rank));
	 * eed_send_command(command_new("(siiii)", "ee_dev_ready_in", device_id, secs, tag, rank));
	 */

	command = command_new("(siiii)", "ee_dev_mode", device_id, mode_id, tag,
			rank);

	ret = command_execute(command, module);

	g_assert_cmpint(ret, ==, TRUE);

	command_unref(command);

	command = command_new("(siiiii)", "ee_dev_mode_in", device_id, mode_id, secs, tag, rank);

	ret = command_execute(command, module);

	g_assert_cmpint(ret, ==, TRUE);

	command_unref(command);

	/* eed_send_command(command_new("(sii)", "ee_finalize", tag, rank)); */

	command = command_new("(si)", "ee_finalize", tag, rank);

	ret = command_execute(command, module);

	g_assert_cmpint(ret, ==, TRUE);

	command_unref(command);

	/* eed_send_command(command_new("(si)", "ee_finalize_all", tag)); */

	command = command_new("(si)", "ee_finalize_all", tag);

	ret = command_execute(command, module);

	g_assert_cmpint(ret, ==, TRUE);

	command_unref(command);

	/* now something that don't exists... */

	command = command_new("(si)",
			"ee_finalize_alljdkjfkdjfkdjkfjdkfjkdjfkdksfjsdk", tag);

	ret = command_execute(command, module);

	g_assert_cmpint(ret, ==, FALSE);

	command_unref(command);

	/* at the moment not testable */
	//	command = command_new("(sii)", "ee_finalize_all", tag);
	//
	//	ret = command_execute(command, module);
	//
	//	g_assert_cmpint(ret, ==, FALSE);
	//
	//	command_unref(command);

	command = command_new("()", "ee_finalize_all", tag);

	ret = command_execute(command, module);

	g_assert_cmpint(ret, ==, FALSE);

	command_unref(command);

	/* at the moment not testable */
	//	command = command_new("(si)", "ee_finalize_all");
	//
	//	ret = command_execute(command, module);
	//
	//	g_assert_cmpint(ret, ==, FALSE);
	//
	//	command_unref(command);

	command = command_new("(s)", "ee_finalize_all");

	ret = command_execute(command, module);

	g_assert_cmpint(ret, ==, FALSE);

	command_unref(command);

	if (!g_module_close(module))
		g_assert_not_reached();

	// wait for all device transitions to finalize...

	sleep(5);
}
