/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * ConstrainedResourceTest.c
 *
 *  Created on: Jul 12, 2011
 *      Author: minartz
 */

#include <glib.h>
#include "Core.h"
#include "ConstrainedResource.h"

void test_constrained_resource_process_list(void) {

	Mode mode;

	gint tag = 1313;
	gint rank = 0;

	Core * core0 = core_new(0);
	core0->startup_time = 200;
	Core * core1 = core_new(1);
	core1->startup_time = 200;

	Resource * resource0 = resource_new(1, core0);
	Resource * resource1 = resource_new(2, core1);

	ConstrainedResource* cr = NULL;

	mode = constrained_resource_process_list(cr, 0);

	g_assert_cmpint(mode, ==, MODE_MAX); // cr == NULL

	cr = constrained_resource_new(0);

	mode = constrained_resource_process_list(cr, 0);

	g_assert_cmpint(mode, ==, MODE_MAX); // no resources registered

	constrained_resource_register(cr, resource0);

	mode = constrained_resource_process_list(cr, 0);

	// no process registered at resource0 -> MODE_UNUSED must be returned
	g_assert_cmpint(mode, ==, MODE_UNUSED);

	resource_register(resource0, tag, rank);

	mode = constrained_resource_process_list(cr, 0);

	// request list is empty -> MODE_MAX must be returned
	g_assert_cmpint(mode, ==, MODE_MAX);

	resource_mode_request(resource0, tag, rank, MODE_MAX);

	mode = constrained_resource_process_list(cr, 0);

	// ony one user who wants MODE_MAX -> MODE_MAX must be returned
	g_assert_cmpint(mode, ==, MODE_MAX);

	constrained_resource_register(cr, resource1);

	resource_mode_request(resource1, tag, rank, MODE_UNUSED);

	mode = constrained_resource_process_list(cr, 0);

	// MODE_MAX + MODE_UNUSED => MODE_MAX
	g_assert_cmpint(mode, ==, MODE_MAX);

	resource_mode_request(resource0, tag, rank, MODE_MIN);

	mode = constrained_resource_process_list(cr, 0);

	// MODE_MIN + MODE_UNUSED => MODE_MIN
	g_assert_cmpint(mode, ==, MODE_MIN);

	resource_unregister(resource0, tag, rank);
	resource_unregister(resource1, tag, rank);

	constrained_resource_unregister(cr, resource0);
	constrained_resource_unregister(cr, resource1);

	constrained_resource_unref(cr);
	resource_unref(resource0);
	resource_unref(resource1);
	core_unref(core0);
	core_unref(core1);

}

void test_constrained_resource_process(void) {

	Mode mode;

	gint tag = 1313;
	gint rank0 = 0, rank1 = 1;

	Core * core0 = core_new(0);
	core0->startup_time = 200;
	Core * core1 = core_new(1);
	core1->startup_time = 200;

	Resource * resource0 = resource_new(1, core0);
	Resource * resource1 = resource_new(2, core1);

	resource_register(resource0, tag, rank0);
	resource_register(resource1, tag, rank1);

	// update every 1 millisecond
	ConstrainedResource* cr = constrained_resource_new(1000);

	constrained_resource_register(cr, resource0);
	constrained_resource_register(cr, resource1);

	mode = constrained_resource_process_list(cr, 0);

	// no request + no request => MODE_MAX
	g_assert_cmpint(mode, ==, MODE_MAX);

	resource_time_request(resource0, MODE_MAX, tag, rank0, 1000000); // resource should be ready in 1 second
	resource_mode_request(resource1, tag, rank1, MODE_UNUSED);

	mode = constrained_resource_process_list(cr, 0);

	// MODE_UNUSED + MODE_UNUSED => MODE_UNUSED
	g_assert_cmpint(mode, ==, MODE_UNUSED);

	// spawn a thread for processing the requests

	if (!g_thread_get_initialized())
		g_thread_init(NULL);

	GThread
			* thread =
					g_thread_create((GThreadFunc) & constrained_resource_process, (gpointer) cr, TRUE, NULL);

	g_usleep(2000000); // sleep two seconds

	cr->finalize = TRUE;

	g_thread_join(thread);

	mode = constrained_resource_process_list(cr, 0);

	// MODE_UNUSED + MODE_MAX => MODE_MAX
	g_assert_cmpint(mode, ==, MODE_MAX);

	constrained_resource_unregister(cr, resource0);
	constrained_resource_unregister(cr, resource1);

	resource_unregister(resource0, tag, rank0);
	resource_unregister(resource1, tag, rank1);

	resource_unref(resource0);
	resource_unref(resource1);

	constrained_resource_unref(cr);

	core_unref(core0);
	core_unref(core1);

}

void test_constrained_resource_cpu(void) {
	Mode mode;

	gint tag = 1313;
	gint rank = 0;
	glong interval = 1000; // update every 1 millisecond

	Core * core = core_new(0);
	core->startup_time = 200;

	g_assert_cmpint(core_get_mode(core), ==, MODE_DEFAULT);

	Resource * resource = resource_new(1, core);

	resource_register(resource, tag, rank);

	ConstrainedResource* cr = constrained_resource_new(interval);

	constrained_resource_register(cr, resource);

	mode = constrained_resource_process_list(cr, 0);

	// no request + no request => MODE_MAX
	g_assert_cmpint(mode, ==, MODE_MAX);

	g_assert_cmpint(core_get_mode(core), ==, MODE_DEFAULT);

	// spawn a thread for processing the requests

	if (!g_thread_get_initialized())
		g_thread_init(NULL);

	GThread
			* thread =
					g_thread_create((GThreadFunc) & constrained_resource_process, (gpointer) cr, TRUE, NULL);

	resource_mode_request(resource, tag, rank, MODE_UNUSED);

	g_usleep(2 * interval);

	g_assert_cmpint(core_get_mode(core), ==, MODE_UNUSED);

	g_assert_cmpint(core_get_state(core), ==, core->frequencies[core->count_frequencies - 1]);

	resource_register(resource, tag, rank + 1);

	g_usleep(2 * interval);

	g_assert_cmpint(core_get_mode(core), ==, MODE_UNUSED);

//	if (core->has_turboboost)
//		g_assert_cmpint(core_get_state(core), ==, core->frequencies[1]);
//	else
//		g_assert_cmpint(core_get_state(core), ==, core->frequencies[0]);

	resource_mode_request(resource, tag, rank + 1, MODE_UNUSED);

	g_usleep(2 * interval);

	g_assert_cmpint(core_get_mode(core), ==, MODE_UNUSED);

	g_assert_cmpint(core_get_state(core), ==, core->frequencies[core->count_frequencies - 1]);

	resource_unregister(resource, tag, rank + 1);

	g_usleep(2 * interval);

	g_assert_cmpint(core_get_mode(core), ==, MODE_UNUSED);

	g_assert_cmpint(core_get_state(core), ==, core->frequencies[core->count_frequencies - 1]);

	cr->finalize = TRUE;

	g_thread_join(thread);

	constrained_resource_unregister(cr, resource);

	resource_unregister(resource, tag, rank);

	resource_unref(resource);

	constrained_resource_unref(cr);

	core_unref(core);

}

/*void test_server(void) {

 gint tag = 1313;
 gint rank1 = 0;
 gint rank2 = 1;

 ee_init(tag, rank1, "prgname");
 ee_init(tag, rank2, "prgname");

 g_usleep(10000);

 ee_dev_ready_in(CORE_ALL_ID, tag, rank1, 20000);
 ee_dev_ready_in(NIC_ALL_ID, tag, rank1, 20000);
 ee_dev_ready_in(DISK_ALL_ID, tag, rank1, 20000);

 g_usleep(10000);

 ee_dev_ready_in(CORE_ALL_ID, tag, rank2, 10000);
 ee_dev_ready_in(NIC_ALL_ID, tag, rank2, 10000);
 ee_dev_ready_in(DISK_ALL_ID, tag, rank2, 10000);

 g_usleep(10000);

 ee_finalize(tag, rank1);
 ee_finalize(tag, rank2);
 }*/
