/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * DiskTest.c
 *
 *  Created on: Jul 13, 2011
 *      Author: minartz
 */

#include <glib.h>

#include "DiskTest.h"
#include "Disk.h"

// only for debugging...
void test_disk_states(void) {

	Disk * disk = disk_new_dev("/dev/sda", 2000000);

	disk_switch_to_mode(disk, MODE_MAX);

	g_assert_cmpint(disk_get_mode(disk), ==, MODE_MAX);

	disk_switch_to_mode(disk, MODE_UNUSED);

	gint counter = 0;
	while (counter < 1000) {
		g_usleep(1000000);
		//		g_print(".");
		g_print("State: %d\n", disk_get_state(disk));
		counter++;
	}

	disk_unref(disk);
}

void test_disk_state_switch(void) {

	glong interval = 2000000; // 2 sekunden

	Disk * disk = disk_new_dev("/dev/sda", interval);

	g_assert_cmpint(disk_get_mode(disk), ==, MODE_DEFAULT);

	disk_switch_to_mode(disk, MODE_MAX);

	g_assert_cmpint(disk_get_mode(disk), ==, MODE_MAX);

	g_assert_cmpint(disk_get_state(disk), ==, STATE_ACTIVE_IDLE);

	disk_switch_to_mode(disk, MODE_MED);

	g_assert_cmpint(disk_get_mode(disk), ==, MODE_MED);

	glong sum = 0;
	State state = disk_get_state(disk);
	while (state != STATE_STANDBY && sum < 1000000) {
		g_usleep(100000);
		sum += 100000;
		state = disk_get_state(disk);
		g_print("State: %d\n", state);
	}

	g_print("Transition duration: %lu microseconds\n", sum);

	g_assert_cmpint(state, ==, STATE_STANDBY);

	disk_switch_to_mode(disk, MODE_MIN);

	g_assert_cmpint(disk_get_mode(disk), ==, MODE_MIN);

	g_usleep(1000000); // sleep one second

	state = disk_get_state(disk);

	g_assert_cmpint(state, ==, STATE_STANDBY);

	disk_switch_to_mode(disk, MODE_UNUSED);

	g_assert_cmpint(disk_get_mode(disk), ==, MODE_UNUSED);

	g_usleep(1000000); // sleep one second

	state = disk_get_state(disk);

	g_assert_cmpint(state, ==, STATE_STANDBY);

	disk_switch_to_mode(disk, MODE_MAX);

	g_assert_cmpint(disk_get_mode(disk), ==, MODE_MAX);

	sum = 0;
	state = disk_get_state(disk);
	while (state != STATE_ACTIVE_IDLE && sum < 1000000) {
		g_usleep(100000);
		sum += 100000;
		state = disk_get_state(disk);
		g_print("State: %d\n", state);
	}

	g_print("Transition duration: %lu microseconds\n", sum);

	g_assert_cmpint(state, ==, STATE_ACTIVE_IDLE);

	disk_unref(disk);

	// MODE_DEFAULT again
}
