/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * InterfaceTest.c
 *
 *  Created on: Aug 6, 2011
 *      Author: Timo Minartz
 */

#include "eed.h"
#include <glib.h>
#include "Command.h"
#include <sys/types.h> /* for sending signals to server process */
#include <signal.h> /* for sending signals to server process */
#include <stdlib.h> /* for getenv */
#include <stdio.h> /* for sscanf */
#include "TestMethods.h"
#include <gio/gio.h>
#include <gio/gunixinputstream.h>

void test_interface_basic(void) {
	int argc = 1;
	gchar * name = "test_interface_basic";
	gchar ** argv = g_malloc(sizeof(gchar*));
	argv[0] = name;
	gint tag = 1313;
	gint rank = 17;
	Command * command = NULL;

	// we need to pin this process...
	gint core_id = pin_process();

	// create server socket

	g_type_init();

	GSocketListener * listen = g_socket_listener_new();

	g_socket_listener_add_inet_port(listen, 1313, NULL, NULL);

	// call interface

	ee_init(&argc, &argv, tag, rank);

	// create connection on server side

	GSocketConnection * conn = g_socket_listener_accept(listen, NULL, NULL,
			NULL);

	g_assert(conn != NULL);

	GInputStream * stream = g_io_stream_get_input_stream((GIOStream*) conn);

	g_assert(stream != NULL);

	gint ret = command_receive(stream, NULL, &command);

	g_assert_cmpint(ret, ==, TRUE);

	GString * result = g_string_new("ee_init");
	g_string_append_printf(result, "(%d,%d,%s,%d)", tag, rank, name, core_id);

	gchar * string = command_get_string(command);

	g_assert_cmpstr(string, ==, result->str);

	command_unref(command);

	g_free(string);

	g_string_free(result, TRUE);

	ee_finalize();

	ret = command_receive(stream, NULL, &command);

	g_assert_cmpint(ret, ==, TRUE);

	result = g_string_new("ee_finalize");
	g_string_append_printf(result, "(%d,%d)", tag, rank);

	string = command_get_string(command);

	g_assert_cmpstr(string, ==, result->str);

	command_unref(command);

	g_free(string);

	g_string_free(result, TRUE);

	g_object_unref(conn);

	g_object_unref(listen);

	g_free(argv);

}

void test_interface_rms(void) {
	int argc = 1;
	gchar * name = "test_interface_rms";
	gchar ** argv = g_malloc(sizeof(gchar*));
	argv[0] = name;
	gint rank = 17;
	Command * command = NULL;

	// we need to pin this process...
	gint core_id = pin_process();

	// create server socket

	g_type_init();

	GSocketListener * listen = g_socket_listener_new();

	g_socket_listener_add_inet_port(listen, 1313, NULL, NULL);

	// validate result received on server side: should be ee_init(tag, rank, name) with tag == PBS_JOBID if a PBS JOB

	gint jobid = 1313;

	gchar *verbstr = getenv("PBS_JOBID");

	if (verbstr != NULL)
		sscanf(verbstr, "%d", &jobid);
	else {
		verbstr = "1313";
		setenv("PBS_JOBID", verbstr, 0);
	}

	// call init from rms
	ee_init_rms_all(&argc, &argv);

	// create connection on server side

	GSocketConnection * conn = g_socket_listener_accept(listen, NULL, NULL,
			NULL);

	g_assert(conn != NULL);

	GInputStream * stream = g_io_stream_get_input_stream((GIOStream*) conn);

	g_assert(stream != NULL);

	// receive command to validate

	gint ret = command_receive(stream, NULL, &command);

	g_assert_cmpint(ret, ==, TRUE);

	GString * result = g_string_new("ee_init");
	g_string_append_printf(result, "(%d,%d,%s,%d)", jobid, RMS_RANK, name, core_id);

	gchar * string = command_get_string(command);

	g_assert_cmpstr(string, ==, result->str);

	command_unref(command);

	g_free(string);

	g_string_free(result, TRUE);

	// call init_rms from application
	ee_init_rms(&argc, &argv, rank);

	ret = command_receive(stream, NULL, &command);

	g_assert_cmpint(ret, ==, TRUE);

	result = g_string_new("ee_init");
	g_string_append_printf(result, "(%d,%d,%s,%d)", jobid, rank, name, core_id);

	string = command_get_string(command);

	g_assert_cmpstr(string, ==, result->str);

	command_unref(command);

	g_free(string);

	g_string_free(result, TRUE);

	// call finalize from application
	ee_finalize();

	ret = command_receive(stream, NULL, &command);

	g_assert_cmpint(ret, ==, TRUE);

	result = g_string_new("ee_finalize");
	g_string_append_printf(result, "(%d,%d)", jobid, rank);

	string = command_get_string(command);

	g_assert_cmpstr(string, ==, result->str);

	command_unref(command);

	g_free(string);

	g_string_free(result, TRUE);

	g_object_unref(conn);

	// call finalize_all from rms
	ee_finalize_all(jobid);

	// recreate connection on server side

	conn = g_socket_listener_accept(listen, NULL, NULL, NULL);

	g_assert(conn != NULL);

	stream = g_io_stream_get_input_stream((GIOStream*) conn);

	g_assert(stream != NULL);

	ret = command_receive(stream, NULL, &command);

	g_assert_cmpint(ret, ==, TRUE);

	result = g_string_new("ee_finalize_all");
	g_string_append_printf(result, "(%d)", jobid);

	string = command_get_string(command);

	g_assert_cmpstr(string, ==, result->str);

	command_unref(command);

	g_free(string);

	g_string_free(result, TRUE);

	g_object_unref(conn);

	g_object_unref(listen);

	g_free(argv);
}

#ifdef HAVE_MPI
#include "mpi.h"

void test_interface_mpi(void) {
	int argc = 1;
	gchar * name = "test_interface_basic";
	gchar ** argv = g_malloc(sizeof(gchar*));
	argv[0] = name;
	gint tag = 1313;
	gint rank = -1;
	Command * command = NULL;

	// create server socket

	g_type_init();

	GSocketListener * listen = g_socket_listener_new();

	g_socket_listener_add_inet_port(listen, 1313, NULL, NULL);

	// initialize MPI
	MPI_Init(& argc, & argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);

	g_assert_cmpint(rank, ==, 0);

	// call interface

	ee_init(&argc, &argv, tag, rank);

	// create connection on server side

	GSocketConnection * conn = g_socket_listener_accept(listen, NULL, NULL,
			NULL);

	g_assert(conn != NULL);

	GInputStream * stream = g_io_stream_get_input_stream((GIOStream*) conn);

	g_assert(stream != NULL);

	gint ret = command_receive(stream, NULL, &command);

	g_assert_cmpint(ret, ==, TRUE);

	GString * result = g_string_new("ee_init");
	g_string_append_printf(result, "(%d,%d,%s)", tag, rank, name);

	gchar * string = command_get_string(command);

	g_assert_cmpstr(string, ==, result->str);

	command_unref(command);

	g_free(string);

	g_string_free(result, TRUE);

	ee_finalize();

	ret = command_receive(stream, NULL, &command);

	g_assert_cmpint(ret, ==, TRUE);

	result = g_string_new("ee_finalize");
	g_string_append_printf(result, "(%d,%d)", tag, rank);

	string = command_get_string(command);

	g_assert_cmpstr(string, ==, result->str);

	command_unref(command);

	g_free(string);

	g_string_free(result, TRUE);

	g_object_unref(conn);

	g_object_unref(listen);

	MPI_Finalize();

	g_free(argv);

}



#endif

/* **************************************************
 * sh_cmd() - executes a command in the background
 * returns TRUE is command was executed  (not the result of the command though..)
 * NO GLOBALS
 */
static gint sh_cmd(gchar * path, gchar * cmd, gchar * args, GPid * pid,
		gint * fd_stderr, gint * fd_stdout) {
	gchar cmd_line[256];
	gchar **argv;
	gint argp;
	gint rc = 0;

	if (cmd == NULL)
		return FALSE;

	if (cmd[0] == '\0')
		return FALSE;

	if (path != NULL)
		chdir(path);

	g_snprintf(cmd_line, sizeof(cmd_line), "%s %s", cmd, args);

	rc = g_shell_parse_argv(cmd_line, &argp, &argv, NULL);
	if (!rc) {
		g_strfreev(argv);
		return rc;
	}

	rc = g_spawn_async_with_pipes(path, argv, NULL, G_SPAWN_SEARCH_PATH, NULL,
			NULL, pid, NULL, fd_stdout, fd_stderr, NULL);

	g_strfreev(argv);

	return rc;
}

void test_interface_application() {

	int argc = 1;
	gchar * name = "test_interface_application";
	gchar ** argv = g_malloc(sizeof(gchar*));
	argv[0] = name;
	gint tag = 1313;
	gint rank = 0;
	gint fd_stderr, fd_stdout;

	// start the server application on this node

	GPid server;

	gboolean ret = sh_cmd(get_working_dir(), "./build/src/server/eeDaemon", "-v",
			&server, &fd_stderr, &fd_stdout);

	g_assert_cmpint(ret, ==, TRUE);

	sleep(2);

	// init eeDaemon

	ee_init(&argc, &argv, tag, rank);

	// finalize eeDaemon

	ee_finalize();

	sleep(2);

	// stop server process
	kill(server, SIGTERM);

	g_spawn_close_pid(server);

	sleep(2);

	// validate server output...

	GInputStream * err = g_unix_input_stream_new(fd_stderr, TRUE);
	GInputStream * out = g_unix_input_stream_new(fd_stdout, TRUE);

	GDataInputStream * err_data = g_data_input_stream_new(err);
	GDataInputStream * out_data = g_data_input_stream_new(out);

	gchar * line = g_data_input_stream_read_line(err_data, NULL, NULL, NULL);

	if(is_test_eeclust_node() && is_test_root_context()) {
		// error log must be empty
		g_assert_cmpstr(line, ==, NULL);
	}

	g_input_stream_close(err, NULL, NULL);
	g_free(line);

	line = g_data_input_stream_read_line(out_data, NULL, NULL, NULL);

	g_assert_cmpstr(line, ==, "[server] Waiting for connection on port 1313...");
	g_free(line);

	line = g_data_input_stream_read_line(out_data, NULL, NULL, NULL);

	while (line != NULL && g_str_has_prefix(line, "[server]")) {
		g_free(line);
		line = g_data_input_stream_read_line(out_data, NULL, NULL, NULL);
	}

	g_assert_cmpstr(line, ==, "Accepted connection on port 1313.");
	g_free(line);

	line = g_data_input_stream_read_line(out_data, NULL, NULL, NULL);

	while (line != NULL && g_str_has_prefix(line, "[server]")) {
		g_free(line);
		line = g_data_input_stream_read_line(out_data, NULL, NULL, NULL);
	}

	g_assert_cmpstr(line, ==, "Rank 0 with tag 1313 initializes Daemon for program test_interface_application.");
	g_free(line);

	line = g_data_input_stream_read_line(out_data, NULL, NULL, NULL);

	while (line != NULL && g_str_has_prefix(line, "[server]")) {
		g_free(line);
		line = g_data_input_stream_read_line(out_data, NULL, NULL, NULL);
	}

	g_assert_cmpstr(line, ==, "Closed connection. Remaining open connections: 0");
	g_free(line);

	g_input_stream_close(out, NULL, NULL);

	g_object_unref(err_data);
	g_object_unref(out_data);
	g_object_unref(err);
	g_object_unref(out);

	g_free(argv);

}

