/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * NicTest.c
 *
 *  Created on: Jul 13, 2011
 *      Author: Timo Minartz
 */

#include <glib.h>

#include "NicTest.h"
#include "Nic.h"

// only for debugging...
void test_nic_states(void) {

	Nic * nic = nic_new_dev("eth0", 2000000);

	nic_switch_to_mode(nic, MODE_MAX);

	g_assert_cmpint(nic_get_mode(nic), ==, MODE_MAX);

	nic_switch_to_mode(nic, MODE_UNUSED);

	gint counter = 0;
	while (counter < 1000) {
		g_usleep(1000000);
		//		g_print(".");
		g_print("State: %d\n", nic_get_state(nic));
		counter++;
	}

	nic_unref(nic);
}

void test_nic_state_switch(void) {

	glong transition_time = 20000000;

	Nic * nic = nic_new_dev("eth0", 2000000);

	g_assert_cmpint(nic_get_mode(nic), ==, MODE_DEFAULT);

	nic_switch_to_mode(nic, MODE_MAX);

	// if test fails here, try: sudo ethtool -s eth0 autoneg on
	g_assert_cmpint(nic_get_mode(nic), ==, MODE_MAX);

	g_assert_cmpint(nic_get_state(nic), ==, STATE_SWITCHING);

	glong sum = 0;
	while (nic_get_state(nic) == STATE_SWITCHING && sum <= 10000000) {
		g_usleep(100000);
		sum += 100000;
	}

	g_print("After %lu microseconds state %d\n", sum, nic_get_state(nic));

	g_assert_cmpint(nic_get_state(nic), ==, 1000);

	nic_switch_to_mode(nic, MODE_MED);

	g_assert_cmpint(nic_get_mode(nic), ==, MODE_MED);

	g_assert_cmpint(nic_get_state(nic), ==, STATE_SWITCHING);

	sum = 0;
	while (nic_get_state(nic) == STATE_SWITCHING && sum <= 10000000) {
		g_usleep(100000);
		sum += 100000;
	}

	g_print("After %lu microseconds state %d\n", sum, nic_get_state(nic));

	g_assert_cmpint(nic_get_state(nic), ==, 100);

	nic_switch_to_mode(nic, MODE_MIN);

	g_assert_cmpint(nic_get_mode(nic), ==, MODE_MIN);

	g_assert_cmpint(nic_get_state(nic), ==, STATE_SWITCHING);

	sum = 0;
	while (nic_get_state(nic) == STATE_SWITCHING && sum <= 10000000) {
		g_usleep(100000);
		sum += 100000;
	}

	g_print("After %lu microseconds state %d\n", sum, nic_get_state(nic));

	g_assert_cmpint(nic_get_state(nic), ==, 10);

	nic_switch_to_mode(nic, MODE_UNUSED);

	g_assert_cmpint(nic_get_mode(nic), ==, MODE_UNUSED);

	sum = 0;
	while (nic_get_state(nic) == STATE_SWITCHING && sum <= 10000000) {
		g_usleep(100000);
		sum += 100000;
	}

	g_print("After %lu microseconds state %d\n", sum, nic_get_state(nic));

	g_assert_cmpint(nic_get_state(nic), ==, 10);

	nic_switch_to_mode(nic, MODE_MAX);

	g_assert_cmpint(nic_get_mode(nic), ==, MODE_MAX);

	sum = 0;
	while (nic_get_state(nic) == STATE_SWITCHING && sum <= 10000000) {
		g_usleep(100000);
		sum += 100000;
	}

	g_print("After %lu microseconds state %d\n", sum, nic_get_state(nic));

	g_assert_cmpint(nic_get_state(nic), ==, 1000);

	nic_unref(nic);

	// MODE_DEFAULT again

	nic = nic_new_dev("eth0", 2000000);

	sum = 0;
	while (nic_get_state(nic) == STATE_SWITCHING && sum <= 10000000) {
		g_usleep(100000);
		sum += 100000;
	}

	g_print("After %lu microseconds state %d\n", sum, nic_get_state(nic));

	g_assert_cmpint(nic_get_state(nic), ==, 1000);

	g_assert_cmpint(nic_get_mode(nic), ==, MODE_DEFAULT);

	nic_unref(nic); // no mode change initiated, because already in MODE_DEFAULT

}
