/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * RMSTest.c
 *
 *  Created on: Nov 9, 2011
 *      Author: Timo Minartz
 */

#include "RMSTest.h"
#include "eed.h"
#include "Core.h"
#include "Server.h"

#include <glib.h>

void test_rms_instrumented_application(void) {

	// All devices should be in MODE_DEFAULT
	// 1. Epilogue calls ee_init_all with jobid as tag
	// All devices should be in MODE_MAX
	// 2. Application calls ee_init_pbs with valid ranks
	// All devices should be in MODE defined by the application
	// 3. Application calls ee_finalize
	// All devices should be in MODE_MAX
	// 4. Prologue calls ee_finalize_all with jobid as tag
	// All devices should be in MODE_UNUSED

	// Initialize one control device, which should point to the core with id 0
	Core * core = core_new(0);

	// start a server

	if (!g_thread_get_initialized())
			g_thread_init(NULL);

	gint port = 1313;
	gint argc = 1;
	gchar ** argv = g_malloc(sizeof(gchar*));
	argv[0] = "test_rms_instrumented_application";

	Server * server = server_new(port);

	GThread *server_thread = g_thread_create((GThreadFunc) &server_run, NULL,
			TRUE, NULL);

	// server running, but no devices instantiated yet -> MODE_DEFAULT
	g_assert_cmpint(core_get_mode(core), ==, MODE_DEFAULT);

	// let rms initialize system
	ee_init_rms_all(&argc, &argv);

	// devices are now instantiated and only one user registered and no requests -> MODE_MAX
	g_assert_cmpint(core_get_mode(core), ==, MODE_MAX);

	// let one rank initialize the system
	ee_init_rms(&argc, &argv, 0);

	// two users registered, still no requests -> MODE_MAX
	g_assert_cmpint(core_get_mode(core), ==, MODE_MAX);

	// request mode MODE_MED for all cpus
	ee_dev_mode(CORE_ALL_ID, MODE_MED);

	// two users registered, one requests MODE_MED -> MODE_MED
	g_assert_cmpint(core_get_mode(core), ==, MODE_MED);

	// finalize from application
	ee_finalize();

	// only one user registered and no requests -> MODE_MAX
	g_assert_cmpint(core_get_mode(core), ==, MODE_MAX);

	// finalize from rms
	ee_finalize_all(0);

	// server running, but no more devices instantiated -> MODE_DEFAULT
	g_assert_cmpint(core_get_mode(core), ==, MODE_DEFAULT);

	// unref test device
	core_unref(core);

	// stop and unref server

	server_stop();

	g_thread_join(server_thread);

	server_unref(server);

	g_free(argv);

}

void test_rms_uninstrumented_application(void) {

	// All devices should be in MODE_UNUSED
	// 1. Epilogue calls ee_init_all with jobid as tag
	// All device should be in MODE_MAX
	// 2. Application runs
	// 3. Prologue calls ee_finalize_all with jobid as tag
	// All devices should be in MODE_UNUSED

}

//void test_resource_management_mixed_applications(void) {
//
//	// 1. Epilogue calls ee_init_all with jobid A as tag
//	// 2. Application A calls ee_init_pbs with valid ranks
//	// 3. Application A runs
//	// 4. Epilogue calls ee_init_all with jobid B as tag
//	// 5. Application B runs
//	// 6. Application A calls ee_finalize
//	// 7. Prologue calls ee_finalize_all with jobid A as tag
//	// 8. Application B is still running
//	// 9. Prologue calls ee_finalize_all with jobid B as tag
//
//}
//
//void test_resource_management_unused_hardware(void) {
//
//	/*
//	 * Assume 24 cores.
//	 * Application A is instrumented and runs on Cores 0-5
//	 * Application B is instrumented and runs on Cores 6-11
//	 * Application C is not instrumented and runs on Cores 12-17
//	 * Cores 18-24 are unused
//	 *
//	 * Problem: How to distinguish groups for Cores 12-17 and 18-24?
//	 */
//
//}

