/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * ResourcesTableTest.c
 *
 *  Created on: Jul 12, 2011
 *      Author: Timo Minartz
 */

#include <glib.h>

#include "ResourcesTable.h"
#include "Resource.h"
#include "Core.h"
#include "Nic.h"
#include "Disk.h"
#include "defines.h"

void test_resources_table(void) {

	gint tag = 1313;
	gint rank0 = 0;
	gint rank1 = 1;
	gint core_id0 = 0;
	gint core_id1 = 1;

	Core * core0 = core_new(0);
	Core * core1 = core_new(1);
	Nic * nic = nic_new();

	Resource * resource0 = resource_new(1, core0);
	Resource * resource1 = resource_new(2, core1);
	Resource * resource2 = resource_new(101, nic);

	ResourcesTable * rt = resources_table_new();

	GList * core_resources = resources_table_get_resources_by_id(rt, 1);

	g_assert_cmpint(g_list_length(core_resources), ==, 0);

	resources_table_add(rt, resource0);
	resources_table_add(rt, resource1);
	resources_table_add(rt, resource2);

	core_resources = resources_table_get_resources_by_id(rt, 1);

	g_assert(
			(Core *) ((Resource *) g_list_first(core_resources)->data)->device == core1);

	g_assert(
			(Core *) (resources_table_get_resource_by_id(rt, 1))->device == core0);

	g_assert(
			(Core *) ((Resource *) g_list_first(core_resources)->next->data)->device == core0);

	g_assert(
			(Core *) (resources_table_get_resource_by_id(rt, 2))->device == core1);

	g_assert_cmpint(resource0->countUsers, ==, 0);
	g_assert_cmpint(resource1->countUsers, ==, 0);
	g_assert_cmpint(resource2->countUsers, ==, 0);

	resources_table_register(rt, tag, rank0, core_id0);

	g_assert_cmpint(resource0->countUsers, ==, 1);
	g_assert_cmpint(resource1->countUsers, ==, 0);
	g_assert_cmpint(resource2->countUsers, ==, 1);

	resources_table_register(rt, tag, rank1, core_id1);

	g_assert_cmpint(resource0->countUsers, ==, 1);
	g_assert_cmpint(resource1->countUsers, ==, 1);
	g_assert_cmpint(resource2->countUsers, ==, 2);

	core_resources = resources_table_get_resources_by_user(rt, tag, rank0);

	g_assert_cmpint(g_list_length(core_resources), ==, 2);

	g_assert(
			(Nic *) ((Resource *) g_list_first(core_resources)->data)->device == nic);

	g_assert(
			(Core *) ((Resource *) g_list_first(core_resources)->next->data)->device == core0);

	core_resources = resources_table_get_resources_by_user(rt, tag, rank1);

	g_assert_cmpint(g_list_length(core_resources), ==, 2);

	g_assert(
			(Core *) ((Resource *) g_list_first(core_resources)->data)->device == core1);

	g_assert(
			(Nic *) ((Resource *) g_list_first(core_resources)->next->data)->device == nic);

	core_resources = resources_table_get_resources_by_user_and_type(rt, tag,
			rank0, MY_CORE_IDS);

	g_assert_cmpint(g_list_length(core_resources), ==, 1);

	g_assert(
			(Core *) ((Resource *) g_list_first(core_resources)->data)->device == core0);

	core_resources = resources_table_get_resources_by_user_and_type(rt, tag,
			rank1, MY_CORE_IDS);

	g_assert_cmpint(g_list_length(core_resources), ==, 1);

	g_assert(
			(Core *) ((Resource *) g_list_first(core_resources)->data)->device == core1);

	resources_table_unregister(rt, tag, rank0);

	g_assert_cmpint(resource0->countUsers, ==, 0);
	g_assert_cmpint(resource1->countUsers, ==, 1);
	g_assert_cmpint(resource2->countUsers, ==, 1);

	resources_table_unregister(rt, tag, rank1);

	g_assert_cmpint(resource0->countUsers, ==, 0);
	g_assert_cmpint(resource1->countUsers, ==, 0);
	g_assert_cmpint(resource2->countUsers, ==, 0);

	resources_table_delete_resource(rt, resource0);

	core_resources = resources_table_get_resources_by_id(rt, 1);

	g_assert_cmpint(g_list_length(core_resources), ==, 1);

	g_assert(
			(Core *) ((Resource *) g_list_first(core_resources)->data)->device == core1);

	g_assert(
			(Core *) (resources_table_get_resource_by_id(rt, 2))->device == core1);

	resources_table_delete_resource(rt, resource1);

	core_resources = resources_table_get_resources_by_id(rt, 1);

	g_assert(g_list_first(core_resources) == NULL);

	core_resources = resources_table_get_resources_by_id(rt, 104);

	g_assert_cmpint(g_list_length(core_resources), ==, 1);

	g_assert(
			(Nic *) ((Resource *) g_list_first(core_resources)->data)->device == nic);

	resource_unref(resource0);
	resource_unref(resource1);
	resource_unref(resource2);

	resources_table_unref(rt);

	core_unref(core0);
	core_unref(core1);
	nic_unref(nic);

}
