/*
 * eeDaemon
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * ServerTest.c
 *
 *  Created on: Jul 13, 2011
 *      Author: Timo Minartz
 */

#ifndef SERVERTEST_C_
#define SERVERTEST_C_

#include "Server.h"
#include "Command.h"
#include "TestMethods.h"
#include "print.h"
#include <glib.h>


void test_server_instantiation(void) {

	Server * server = server_new(1313);

	g_assert_cmpint(server->instances, ==, 1);

	Server * server2 = server_new(1313);

	g_assert_cmpint(server->instances, ==, 2);
	g_assert(server == server2);

	Server * server3 = server_new(1314);

	g_assert_cmpint(server->instances, ==, 2);
	g_assert(server3 == NULL);

	server_unref(server2);

	g_assert_cmpint(server->instances, ==, 1);

	g_assert(server != NULL);

	server_unref(server);

}

void test_server_run(void) {

	gint tag = 1313;
	gint rank = 0;
	gint port = 1313;

	if (!g_thread_get_initialized())
		g_thread_init(NULL);

	// create server

	Server * server = server_new(port);

	GThread *server_thread =
			g_thread_create((GThreadFunc) & server_run, NULL, TRUE, NULL);

	// create client

	GSocketClient* client = g_socket_client_new();

	GSocketConnection* conn = g_socket_client_connect_to_host(client,
			"localhost", port, NULL, NULL);

	g_assert(conn != NULL);

	GOutputStream* stream = g_io_stream_get_output_stream((GIOStream*) conn);

	g_assert(stream != NULL);

	Command * command = command_new("(siis)", "ee_init", tag, rank, "test_server_run");

	gboolean ret = command_send(stream, command);

	command_unref(command);

	g_assert_cmpint(ret, ==, TRUE);

	command = command_new("(sii)", "ee_finalize", tag, rank);

	ret = command_send(stream, command);

	command_unref(command);

	g_assert_cmpint(ret, ==, TRUE);

	// if on an eeclust node, client should be terminated now. But server is still waiting for input connection...

	server_stop();

	g_thread_join(server_thread);

	g_object_unref(conn);

	g_object_unref(client);

	server_unref(server);

}

void test_server_run_multiple_clients(void) {

	gint tag = 1313;
	gint rank = 0;
	gint port = 1313;

	if (!g_thread_get_initialized())
		g_thread_init(NULL);

	// create server

	Server * server = server_new(port);

	GThread *server_thread =
			g_thread_create((GThreadFunc) & server_run, NULL, TRUE, NULL);

	// create client and streams

	GSocketClient* client[3];
	GSocketConnection* conn[3];
	GOutputStream* stream[3];

	for (gint i = 0; i < 3; ++i) {
		client[i] = g_socket_client_new();

		GError * error = NULL;
		conn[i] = g_socket_client_connect_to_host(client[i], "localhost", port,
				NULL, &error);

		if(conn[i] == NULL) {
			error("Connection failed: %s\n", error->message);
		}

		g_assert(conn[i] != NULL);

		stream[i] = g_io_stream_get_output_stream((GIOStream*) conn[i]);

		g_assert(stream[i] != NULL);

	}


	// send commands

	for (gint i = 0; i < 3; ++i) {

		Command * command = command_new("(siisi)", "ee_init", tag, rank+i, "test_server_run_multiple_clients", 0);

		gboolean ret = command_send(stream[i], command);

		command_unref(command);

		g_assert_cmpint(ret, ==, TRUE);

	}

	// wait for possible state transitions
	sleep(5);

	for (gint i = 2; i >= 0; --i) {

		Command * command = command_new("(sii)", "ee_finalize", tag, rank+i);

		gboolean ret = command_send(stream[i], command);

		command_unref(command);

		g_assert_cmpint(ret, ==, TRUE);
	}

	// wait for possible state transitions
	sleep(5);

	// if on an eeclust node, all clients should be terminated now

	server_stop();

	for (gint i = 0; i < 3; ++i) {
		g_object_unref(conn[i]);

		g_object_unref(client[i]);
	}

	g_thread_join(server_thread);

	server_unref(server);

}

void test_server_termination(void) {

	gint port = 1313;

	Server * server = server_new(port);

	GThread *server_thread =
			g_thread_create((GThreadFunc) & server_run, NULL, TRUE, NULL);

	raise(SIGTERM);

	g_thread_join(server_thread);

	server_unref(server);

	server = server_new(port);

	server_thread
			= g_thread_create((GThreadFunc) & server_run, NULL, TRUE, NULL);

	raise(SIGTERM);

	g_thread_join(server_thread);

	server_unref(server);

	server = server_new(port);

	server_thread
			= g_thread_create((GThreadFunc) & server_run, NULL, TRUE, NULL);

	raise(SIGINT);

	g_thread_join(server_thread);

	server_unref(server);
}

#endif /* SERVERTEST_C_ */
